@@ -14,11 +14,11 @@ Text::Amuse - Perl module to generate HTML and LaTeX documents from Emacs Muse m
 
 =head1 VERSION
 
-Version 0.52
+Version 0.53
 
 =cut
 
-our $VERSION = '0.52';
+our $VERSION = '0.53';
 
 
 =head1 SYNOPSIS
@@ -214,8 +214,31 @@ Return the HTML formatted ToC, as a string.
 
 sub toc_as_html {
     my $self = shift;
-    $self->as_html; # be sure that it's processed
-    return $self->_html_obj->html_toc;
+    my @toc = $self->raw_html_toc;
+    return "" unless @toc;
+    # do the dirty job
+    my @out;
+    foreach my $item (@toc) {
+        next unless $item->{index}; # skip the 0 one, is dummy
+
+        # given that we wrap this into <a>, we have to strip eventual ones
+        my $string = $item->{string};
+        $string =~ s!<a [^>]+?>!!g;
+        $string =~ s!</a>!!g;
+
+        my $line = qq{<p class="tableofcontentline toclevel} .
+          $item->{level} . qq{"><span class="tocprefix">} .
+          "&nbsp;&nbsp;" x  $item->{level} . "</span>" .
+            qq{<a href="#toc} . $item->{index} . qq{">} .
+              $string . "</a></p>";
+        push @out, $line;
+    }
+    if (@out) {
+        return join ("\n", @out) . "\n";
+    }
+    else {
+        return '';
+    }
 }
 
 =head3 as_splat_html
